import { Selector, createSelector } from '@ngrx/store';
import { EntityState, createEntityAdapter, Dictionary } from '@ngrx/entity';

import { InfoResponse, Info, EventJson } from '@ucap/protocol-event';

import { FileInfo, FileDownloadInfo } from '@ucap/protocol-file';

export interface EventListState extends EntityState<Info<EventJson>> {}
export interface FileInfoListState extends EntityState<FileInfo> {}
export interface FileInfoCheckListState extends EntityState<FileDownloadInfo> {}

export const adapterEventList = createEntityAdapter<Info<EventJson>>({
  selectId: info => info.seq,
  sortComparer: (a, b) => {
    return a.seq - b.seq;
  }
});
export const adapterFileInfoList = createEntityAdapter<FileInfo>({
  selectId: info => info.seq,
  sortComparer: (a, b) => {
    return b.seq - a.seq;
  }
});
export const adapterFileInfoCheckList = createEntityAdapter<FileDownloadInfo>({
  selectId: info => info.seq,
  sortComparer: (a, b) => {
    return b.seq - a.seq;
  }
});

const eventListInitialState: EventListState = adapterEventList.getInitialState(
  {}
);
const fileInfoListInitialState: FileInfoListState = adapterFileInfoList.getInitialState(
  {}
);
const fileInfoCheckListInitialState: FileInfoCheckListState = adapterFileInfoCheckList.getInitialState(
  {}
);

const {
  selectAll: selectAllForEventList,
  selectEntities: selectEntitiesForEventList,
  selectIds: selectIdsForEventList,
  selectTotal: selectTotalForEventList
} = adapterEventList.getSelectors();

const {
  selectAll: selectAllForFileInfoList,
  selectEntities: selectEntitiesForFileInfoList,
  selectIds: selectIdsForFileInfoList,
  selectTotal: selectTotalForFileInfoList
} = adapterFileInfoList.getSelectors();

const {
  selectAll: selectAllForFileInfoCheckList,
  selectEntities: selectEntitiesForFileInfoCheckList,
  selectIds: selectIdsForFileInfoCheckList,
  selectTotal: selectTotalForFileInfoCheckList
} = adapterFileInfoCheckList.getSelectors();

export interface Chatting {
  roomId?: string;

  eventListProcessing?: boolean;
  eventList?: EventListState;
  eventStatus?: InfoResponse | null;
  remainEvent?: false;

  fileInfoListProcessing?: boolean;
  fileInfoList?: FileInfoListState;
  fileInfoCheckList?: FileInfoCheckListState;
  fileInfoSyncDate?: string;
}

export interface ChattingState extends EntityState<Chatting> {}

export const adapterChatting = createEntityAdapter<Chatting>({
  selectId: chatting => chatting.roomId
});

const {
  selectAll: selectAllForChatting,
  selectEntities: selectEntitiesForChatting,
  selectIds: selectIdsForChatting,
  selectTotal: selectTotalForChatting
} = adapterChatting.getSelectors();

const chattingInitialState: ChattingState = adapterChatting.getInitialState({
  roomId: undefined,

  eventListProcessing: false,
  eventList: eventListInitialState,
  eventStatus: undefined,
  remainEvent: false,

  fileInfoListProcessing: false,
  fileInfoList: fileInfoListInitialState,
  fileInfoCheckList: fileInfoCheckListInitialState,
  fileInfoSyncDate: undefined
});

export interface State {
  chattings: ChattingState;
}

export const initialState: State = {
  chattings: chattingInitialState
};

export function selectors<S>(selector: Selector<any, State>) {
  const selectChattings = createSelector(
    selector,
    (state: State) => state.chattings
  );

  const selectChatting = createSelector(
    selector,
    selectEntitiesForChatting,
    (state: State, entities: Dictionary<Chatting>, roomId: string) =>
      entities && entities[roomId]
  );

  const selectChattingEventListProcessing = createSelector(
    selectChatting,
    state => state.eventListProcessing
  );
  const selectChattingEventList = createSelector(
    selectChatting,
    state => state.eventList
  );
  const selectChattingEventStatus = createSelector(
    selectChatting,
    state => state.eventStatus
  );
  const selectChattingRemainEvent = createSelector(
    selectChatting,
    state => state.remainEvent
  );
  const selectChattingFileInfoListProcessing = createSelector(
    selectChatting,
    state => state.fileInfoListProcessing
  );
  const selectChattingFileInfoList = createSelector(
    selectChatting,
    state => state.fileInfoList
  );
  const selectChattingFileInfoCheckList = createSelector(
    selectChatting,
    state => state.fileInfoCheckList
  );
  const selectChattingFileInfoSyncDate = createSelector(
    selectChatting,
    state => state.fileInfoSyncDate
  );

  return {
    chattings: createSelector(selectChattings, selectAllForChatting),
    chatting: selectChatting,

    eventListProcessing: selectChattingEventListProcessing,
    eventList: createSelector(selectChattingEventList, selectAllForEventList),
    eventStatus: selectChattingEventStatus,
    remainEvent: selectChattingRemainEvent,

    fileInfoListProcessing: selectChattingFileInfoListProcessing,
    fileInfoList: createSelector(
      selectChattingFileInfoList,
      selectAllForFileInfoList
    ),
    fileInfoCheckList: createSelector(
      selectChattingFileInfoCheckList,
      selectAllForFileInfoCheckList
    ),
    fileInfoSyncDate: selectChattingFileInfoSyncDate
  };
}
